﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace DynamicLoad
{
    //方法调用器
    /// <summary>
    /// 方法调用器
    /// </summary>
    public class MethodInvoker : MarshalByRefObject,IDisposable
    {
        //程序集名称
        private readonly string _DllName;
        //程序集中的类类型全名
        private readonly string _TypeName;
        //方法信息缓存列表
        private readonly Dictionary<string, MethodInfo> _Methods = 
            new Dictionary<string, MethodInfo>();
        //程序集中的类类型实例
        private object _TypeInstance;

        //构造方法
        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="dllName"></param>
        /// <param name="typeName"></param>
        public MethodInvoker(string dllName, string typeName)
        {
            this._DllName = dllName;
            this._TypeName = typeName;
        }

        //加载程序集中的所有方法
        /// <summary>
        /// 加载程序集中的所有方法
        /// </summary>
        public void LoadAllMethods()
        {
            Assembly assembly = Assembly.LoadFrom(_DllName);
            if (assembly == null)
                throw new Exception("Can't find " + _DllName);
            Type tp = assembly.GetType(_TypeName);
            if (tp == null)
                throw new Exception("Can't get type " + _TypeName + " from " + _DllName);
            _TypeInstance = Activator.CreateInstance(tp);
            if (_TypeInstance == null)
                throw new Exception("Can't construct type " + _TypeName + " from " + _DllName);

            MethodInfo[] typeMethod;

            if (_Methods.Count == 0)
            {
                typeMethod = tp.GetMethods(BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.Instance);

                for (int i = 0; i < typeMethod.Length; i++)
                {
                    if (typeMethod[i].DeclaringType != typeof(object))
                    {
                        MethodInfo method;
                        if (!_Methods.TryGetValue(typeMethod[i].Name, out method))
                            _Methods.Add(typeMethod[i].Name, typeMethod[i]);
                    }
                }
            }
        }

        //调用程序集中的指定方法
        /// <summary>
        /// 调用程序集中的指定方法
        /// </summary>
        /// <param name="methodName">方法名称</param>
        /// <param name="methodParams">参数数组</param>
        /// <returns></returns>
        public object InvokeMethod(string methodName, object[] methodParams)
        {
            MethodInfo method;
            if (string.IsNullOrEmpty(methodName))
                throw new Exception("Method Name IsNullOrEmpty");

            _Methods.TryGetValue(methodName, out method);

            if (method == null)
                throw new Exception("Method can not be found");

            object result = method.Invoke(_TypeInstance, methodParams);
            //这里可以对result进行包装
            return result;
        }


        #region IDisposable 成员

        public void Dispose()
        {
            _TypeInstance = null;
            GC.Collect();
            GC.WaitForPendingFinalizers();
            GC.Collect(0);
        }
        #endregion
    }
}
